@@ -2,9 +2,9 @@
  * 
  * fitqtl_imp.c
  *
- * copyright (c) 2002, Hao Wu, The Jackson Laboratory
+ * copyright (c) 2002-3, Hao Wu, The Jackson Laboratory
  *
- * last modified June, 2002
+ * last modified Dec, 2003
  * first written Apr, 2002
  *
  * Licensed under the GNU General Public License version 2 (June, 1991)
@@ -317,8 +317,8 @@ double galtRss(double *pheno, int n_ind, int *n_gen, int n_qtl,
 
 
   /* call dqrls to fit regression model */
-  dqrls_(x, &n_ind, &sizefull, pheno, &ny, &tol, coef, resid,
-         qty, &k, jpvt, qraux, work);
+  F77_CALL(dqrls)(x, &n_ind, &sizefull, pheno, &ny, &tol, coef, resid,
+		  qty, &k, jpvt, qraux, work);
 
   /* calculate RSS */
   for(i=0; i<n_ind; i++)
