@@ -2,9 +2,9 @@
  * 
  * hmm_4way.c
  * 
- * copyright (c) 2001, Karl W Broman, Johns Hopkins University
+ * copyright (c) 2001, 2002, Karl W Broman, Johns Hopkins University
  * 
- * last modified Nov, 2001
+ * last modified Mar, 2002
  * first written Feb, 2001
  *
  * Licensed under the GNU General Public License version 2 (June, 1991)
@@ -95,7 +95,7 @@ double emit_4way(int obs_gen, int true_gen, double error_prob)
     case 1: case 4: return(log(2.0*error_prob/3.0));
     }
   }
-  return(log(-1.0)); /* shouldn't get here */
+  return(0.0); /* shouldn't get here */
 }
 
 double step_4way(int gen1, int gen2, double rf1, double rf2)
