@@ -2,9 +2,9 @@
  * 
  * scanone_em.c
  *
- * copyright (c) 2001, Karl W Broman, Johns Hopkins University
+ * copyright (c) 2001-2, Karl W Broman, Johns Hopkins University
  *
- * last modified Nov, 2001
+ * last modified Oct, 2002
  * first written Nov, 2001
  *
  * Licensed under the GNU General Public License version 2 (June, 1991)
@@ -41,6 +41,7 @@
 void R_scanone_em(int *n_ind, int *n_pos, int *n_gen, 
 		  double *genoprob, double *addcov, int *n_addcov,
 		  double *intcov, int *n_intcov, double *pheno,
+		  double *weights,
 		  double *result, int *std_start, double *start,
 		  int *maxit, double *tol, int *trace)
 {
@@ -58,16 +59,16 @@ void R_scanone_em(int *n_ind, int *n_pos, int *n_gen,
     /* Read R's random seed */
     GetRNGstate();
 
-    scanone_em(*n_ind, *n_pos, *n_gen, Genoprob, pheno, Result, 
-	       *std_start, start, *maxit, *tol, work);
+    scanone_em(*n_ind, *n_pos, *n_gen, Genoprob, pheno, weights, 
+	       Result, *std_start, start, *maxit, *tol, work);
 
     /* Write R's random seed */
     PutRNGstate();
   }
   else { /* interval mapping with covariates */
     scanone_em_covar(*n_ind, *n_pos, *n_gen, Genoprob, Addcov,
-		     *n_addcov, Intcov, *n_intcov, pheno, result,
-		     *maxit, *tol, *trace);
+		     *n_addcov, Intcov, *n_intcov, pheno, weights,
+		     result, *maxit, *tol, *trace);
   }
 }
 
@@ -89,6 +90,8 @@ void R_scanone_em(int *n_ind, int *n_pos, int *n_gen,
  *
  * pheno        Phenotype data, as a vector
  *
+ * weights      Vector of positive weights, of length n_ind
+ *
  * Result       Result matrix of size [n_pos x (n_gen+2)]; upon return, 
  *              the first column contains the log10 likelihood, the 
  *              next set contain estimated genotype-specific means, and 
@@ -112,13 +115,17 @@ void R_scanone_em(int *n_ind, int *n_pos, int *n_gen,
  **********************************************************************/
 
 void scanone_em(int n_ind, int n_pos, int n_gen, double ***Genoprob,
-		double *pheno, double **Result, int std_start, 
-                double *start,
+		double *pheno, double *weights, 
+		double **Result, int std_start, double *start,
 		int maxit, double tol, double **work)
 {
   int i, j, k, s, flag=0;
   double s1, s2, s3, oldsig, r;
 
+  /* turn weights back into usual scale rather than sqrt(weights) */
+  for(j=0; j<n_ind; j++) 
+    weights[j] *= weights[j];
+
   for(i=0; i<n_pos; i++) { /* loop over marker positions */
 
     /* initiate EM */
@@ -133,28 +140,28 @@ void scanone_em(int n_ind, int n_pos, int n_gen, double ***Genoprob,
 	for(k=0; k<n_gen; k++) {
 	  work[1][k] = s2 = s3 = 0.0;
 	  for(j=0; j<n_ind; j++) {
-	    s2 += Genoprob[k][i][j]; /* count up numbers */
-	    work[1][k] += Genoprob[k][i][j]*pheno[j]; /* means */
-	    s3 += Genoprob[k][i][j]*pheno[j]*pheno[j]; /* RSS */
+	    s2 += Genoprob[k][i][j]*weights[j]; /* count up numbers */
+	    work[1][k] += Genoprob[k][i][j]*pheno[j]*weights[j]; /* means */
+	    s3 += Genoprob[k][i][j]*pheno[j]*pheno[j]*weights[j]; /* for RSS */
 	  }
-	  s1 += (s3 - work[1][k]*work[1][k]/s2);
+	  s1 += (s3 - work[1][k]*work[1][k]/s2); /* RSS */
 	  work[1][k] /= s2;
 	}
-	oldsig = sqrt(s1/(double)(n_ind));
+	oldsig = sqrt(s1/(double)n_ind);
       }
       else { /* start using random weights */
 	for(k=0; k<n_gen; k++) {
 	  work[1][k] = s2 = s3 = 0.0;
 	  for(j=0; j<n_ind; j++) {
 	    r = unif_rand()/(double)(n_gen); 
-	    s2 += r; /* count up numbers */
-	    work[1][k] += r*pheno[j]; /* means */
-	    s3 += r*pheno[j]*pheno[j]; /* RSS */
+	    s2 += r*weights[j]; /* count up numbers */
+	    work[1][k] += r*pheno[j]*weights[j]; /* means */
+	    s3 += r*pheno[j]*pheno[j]*weights[j]; /* RSS */
 	  }
 	  s1 += (s3 - work[1][k]*work[1][k]/s2);
 	  work[1][k] /= s2;
 	}
-	oldsig = sqrt(s1/(double)(n_ind));
+	oldsig = sqrt(s1/(double)n_ind);
       }
     }
 
@@ -169,15 +176,15 @@ void scanone_em(int n_ind, int n_pos, int n_gen, double ***Genoprob,
 	s1=0.0;
 	for(k=0; k<n_gen; k++) 
 	  s1 += (work[0][k] = Genoprob[k][i][j]*
-		 dnorm(pheno[j],work[1][k],oldsig,0));
+		 dnorm(pheno[j],work[1][k],oldsig/sqrt(weights[j]),0));
 	for(k=0; k<n_gen; k++) 
 	  work[0][k] /= s1;
 	
 	/* M-step */
 	for(k=0; k<n_gen; k++) {
-	  work[2][k] += work[0][k]; /* count up numbers */
-	  Result[k+1][i] += work[0][k] * pheno[j]; /* means */
-	  work[3][k] += work[0][k] * pheno[j] * pheno[j]; /* RSS */
+	  work[2][k] += work[0][k]*weights[j]; /* count up numbers */
+	  Result[k+1][i] += work[0][k] * pheno[j]*weights[j]; /* means */
+	  work[3][k] += work[0][k] * pheno[j] * pheno[j]*weights[j]; /* RSS */
 	}
       }
       
@@ -214,7 +221,7 @@ void scanone_em(int n_ind, int n_pos, int n_gen, double ***Genoprob,
       s1 = 0.0;
       for(k=0; k<n_gen; k++) 
 	s1 += Genoprob[k][i][j] * dnorm(pheno[j], Result[1+k][i], 
-					Result[1+n_gen][i], 0);
+					Result[1+n_gen][i]/sqrt(weights[j]), 0);
       Result[0][i] -= log10(s1);
     }
     Result[1+n_gen][i] *= sqrt((double)n_ind/(double)(n_ind-n_gen));
